@@ -4,7 +4,7 @@
  * Implementation of the part of the TclCurl extension that deals with libcurl's
  * 'multi' interface.
  *
- * Copyright (c)2002-2003 Andrs Garca Garca.
+ * Copyright (c)2002-2004 Andrs Garca Garca.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -180,7 +180,7 @@ curlMultiObjCmd (ClientData clientData, Tcl_Interp *interp,
             return errorCode;
             break;
         case 3:
-/*            fprintf(stdout,"Multi cleanup\n"); */
+            fprintf(stdout,"Multi cleanup\n");
             Tcl_DeleteCommandFromToken(interp,curlMultiData->token);
             break;
         case 4:
@@ -192,8 +192,8 @@ curlMultiObjCmd (ClientData clientData, Tcl_Interp *interp,
             curlMultiActiveTransfers(interp,curlMultiData);
             break;
         case 6:
-            fprintf(stdout,"Multi auto transfer\n");
-            curlMultiAutoTransfer(interp,curlMultiData);
+/*            fprintf(stdout,"Multi auto transfer\n");*/
+            curlMultiAutoTransfer(interp,curlMultiData,objc,objv);
             break;
     }
     return TCL_OK;
@@ -347,6 +347,7 @@ curlMultiDeleteCmd(ClientData clientData) {
         listPtr1=listPtr2;
     }
     errorCode=curl_multi_cleanup(curlMultiHandle);
+    curlMultiFreeSpace(curlMultiData);
     return curlReturnCURLMcode(interp,errorCode);
 }
 
@@ -404,8 +405,7 @@ curlMultiGetInfo(Tcl_Interp *interp,CURLM *curlMultiHandlePtr) {
  *    multiHandlePtr: Pointer to the multi handle of the transfer.
  *
  * Results:
- *    Standard Tcl codes. The Tcl command will return the number of
- *    transfers.
+ *    
  *----------------------------------------------------------------------
  */
 int
@@ -516,10 +516,11 @@ curlGetEasyHandle(Tcl_Interp *interp,Tcl_Obj *nameObjPtr) {
  *----------------------------------------------------------------------
  */
 void
-curlMultiFreeSpace(CURLM *curlMultiHandle,struct curlMultiObjData *curlMultiData) {
+curlMultiFreeSpace(struct curlMultiObjData *curlMultiData) {
 
-    curl_multi_cleanup(curlMultiHandle);
+    curl_multi_cleanup(curlMultiData->mcurl);
 
+    Tcl_Free(curlMultiData->postCommand);
     Tcl_Free((char *)curlMultiData);
 }
 
@@ -653,7 +654,7 @@ curlReturnCURLMcode (Tcl_Interp *interp,CURLMcode errorCode) {
 
 /*----------------------------------------------------------------------
  *
- * curlCreateEventSource --
+ * curlMultiAutoTransfer --
  *
  *	Creates the event source that will take care of downloading using
  *  the multi interface driven by Tcl's event loop.
@@ -667,25 +668,63 @@ curlReturnCURLMcode (Tcl_Interp *interp,CURLMcode errorCode) {
  */
 
 int
-curlMultiAutoTransfer(Tcl_Interp *interp, struct curlMultiObjData *curlMultiData) {
-fprintf(stdout,"Llegamos a crear la fuente de eventos\n");
+curlMultiAutoTransfer(Tcl_Interp *interp, struct curlMultiObjData *curlMultiData,
+        int objc,Tcl_Obj *CONST objv[]) {
 
-    Tcl_CreateEventSource((Tcl_EventSetupProc *)NULL, 
+    if (objc==4) {
+        Tcl_Free(curlMultiData->postCommand);
+        curlMultiData->postCommand=curlstrdup(Tcl_GetString(objv[3]));
+    }
+
+    Tcl_CreateEventSource((Tcl_EventSetupProc *)curlEventSetup, 
             (Tcl_EventCheckProc *)curlEventCheck, (ClientData *)curlMultiData);
 
     /* We have to call perform once to boot the transfer, otherwise it seems nothing
        works *shrug* */
 
-      while(CURLM_CALL_MULTI_PERFORM ==
-            curl_multi_perform(curlMultiData->mcurl,&(curlMultiData->runningTransfers)));
+    while(CURLM_CALL_MULTI_PERFORM ==
+            curl_multi_perform(curlMultiData->mcurl,&(curlMultiData->runningTransfers))) {
+    }
 
     return TCL_OK;
 }
 
+/*----------------------------------------------------------------------
+ *
+ * curlEventSetup --
+ *
+ *	This function is invoked by Tcl just after curlMultiAutoTransfer and
+ *  then every time just before curlEventCheck, I only use to set the
+ *  maximun time without checking for events
+ *
+ *  NOTE: I hate having a fixed value, I will have to look into it.
+ *
+ * Parameters:
+ *  They are passed automagically by Tcl, but I don't use them.
+ *----------------------------------------------------------------------
+ */
+
 void
 curlEventSetup(ClientData clientData, int flags) {
+    Tcl_Time     time = {0 , 0};
+
+    Tcl_SetMaxBlockTime(&time);
 }
 
+/*----------------------------------------------------------------------
+ *
+ * curlEventCheck --
+ *
+ *	Invoked automagically by Tcl from time to time, we check if there
+ *  are any active transfer, if so we put an event in the queue so that
+ *  'curl_multi_perfom' will be eventually called, if not we delete
+ *  the event source.
+ *
+ * Parameters:
+ *  They are passed automagically by Tcl.
+ *----------------------------------------------------------------------
+ */
+
 void
 curlEventCheck(ClientData clientData, int flags) {
     struct curlMultiObjData    *curlMultiData=(struct curlMultiObjData *)clientData;
@@ -693,29 +732,53 @@ curlEventCheck(ClientData clientData, int flags) {
     struct curlEvent           *curlEventPtr;
     int                         selectCode;
 
-    /* Lo primero mirar si hay transferencias activas. */
     selectCode=curlMultiGetActiveTransfers(curlMultiData);
 
     if (curlMultiData->runningTransfers==0) {
-        Tcl_DeleteEventSource((Tcl_EventSetupProc *)NULL, 
+        Tcl_DeleteEventSource((Tcl_EventSetupProc *)curlEventSetup, 
                 (Tcl_EventCheckProc *)curlEventCheck, (ClientData *)curlMultiData);
     } else {
-        if (selectCode>0) {
-        } else {fprintf(stdout,"No hay transferencias activas\n");}
+        if (selectCode>=0) {
             curlEventPtr=(struct curlEvent *)Tcl_Alloc(sizeof(struct curlEvent));
-            curlEventPtr->proc=curlFakeEventProc;
+            curlEventPtr->proc=curlEventProc;
             curlEventPtr->curlMultiData=curlMultiData;
             Tcl_QueueEvent((Tcl_Event *)curlEventPtr, TCL_QUEUE_TAIL);
+        }
     }
 }
 
+/*----------------------------------------------------------------------
+ *
+ * curlEventProc --
+ *
+ *	Finally Tcl event loop decides it is time to transfer something.
+ *
+ * Parameters:
+ *  They are passed automagically by Tcl.
+ *----------------------------------------------------------------------
+*/ 
+
 int
-curlFakeEventProc(Tcl_Event *evPtr,int flags) {
-    struct curlMultiObjData   *curlMultiData=(struct curlMultiObjData *)((struct curlEvent *)evPtr)->curlMultiData;
-    CURLMcode        errorCode;
+curlEventProc(Tcl_Event *evPtr,int flags) {
+    struct curlMultiObjData   *curlMultiData
+            =(struct curlMultiObjData *)((struct curlEvent *)evPtr)->curlMultiData;
+    CURLMcode                  errorCode;
+    Tcl_Obj                   *tclCommandObjPtr;
+    char                       tclCommand[300];
 
     errorCode=curl_multi_perform(curlMultiData->mcurl,&curlMultiData->runningTransfers);
-
+    if (curlMultiData->runningTransfers==0) {
+        if (curlMultiData->postCommand!=NULL) {
+            snprintf(tclCommand,299,"%s",curlMultiData->postCommand);
+            tclCommandObjPtr=Tcl_NewStringObj(tclCommand,-1);
+            if (Tcl_EvalObjEx(curlMultiData->interp,tclCommandObjPtr,TCL_EVAL_GLOBAL)!=TCL_OK) {
+/*
+                fprintf(stdout,"Error invoking command\n");
+                fprintf(stdout,"Error: %s\n",Tcl_GetString(Tcl_GetObjResult(curlMultiData->interp)));
+*/
+            }
+        }
+    }
     return 1;
 }
 
