@@ -4,7 +4,7 @@
  * Header file for the part of the TclCurl extension that deals with libcurl's
  * 'multi' interface.
  *
- * Copyright (c) 2002-2003 Andrs Garca Garca.
+ * Copyright (c) 2002-2004 Andrs Garca Garca.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -33,6 +33,7 @@ struct curlMultiObjData {
     fd_set                 fdwrite;
     fd_set                 fdexcep;
     int                    runningTransfers;
+    char                  *postCommand;    
 };
 
 struct curlEvent {
@@ -79,7 +80,7 @@ int curlMultiActiveTransfers(Tcl_Interp *interp, struct curlMultiObjData *curlMu
 
 struct curlObjData *curlGetEasyHandle(Tcl_Interp *interp,Tcl_Obj *nameObjPtr);
 
-void curlMultiFreeSpace(CURLM *curlMultiHandle,struct curlMultiObjData *curlMultiData);
+void curlMultiFreeSpace(struct curlMultiObjData *curlMultiData);
 
 int curlReturnCURLMcode(Tcl_Interp *interp,CURLMcode errorCode);
 
@@ -88,13 +89,13 @@ void curlEasyHandleListRemove(struct curlMultiObjData *multiDataPtr,CURL *easyHa
 char *curlGetEasyName(struct curlMultiObjData *multiDataPtr,CURL *easyHandle);
 
 
-int curlMultiAutoTransfer(Tcl_Interp *interp, struct curlMultiObjData *curlMultiData);
+int curlMultiAutoTransfer(Tcl_Interp *interp, struct curlMultiObjData *curlMultiData, int objc,Tcl_Obj *CONST objv[]);
 
 void curlEventSetup(ClientData clientData, int flags);
 
 void curlEventCheck(ClientData clientData, int flags);
 
-int curlFakeEventProc(Tcl_Event *evPtr,int flags);
+int curlEventProc(Tcl_Event *evPtr,int flags);
 
 #ifdef  __cplusplus
 }
